package sinhalatts_vbeta;

/** This can analize the Uniocde format tokens */
/** Author: Viraj Welgama
 * LTRL - UCSC
 * March 2005
 */
import java.util.ArrayList;
import java.util.regex.*;

class TextNormalizer {

  private ArrayList Acronyms;
  private ArrayList AcronymsExtract;
  private ArrayList Units;
  private ArrayList UnitsDef;
  private String AcronymsFilePath = ".\\Resources\\Acronyms.res";
  private String UnitsFilePath = ".\\Resources\\Units.res";

  private boolean foundAUnit = false;

  private final String SCHWA_SOUND = "axz";

  private final String PhoneSet[] = {
      "q", "h", "", "a", "aaz", "aez", "awz", "i", "iiz", "u", "uuz", "r i",
      "r iiz",
      "i l u", "i l uuz", "e", "eez", "aiz", "o", "ooz", "auz", "", "", "", "k",
      "k", "g", "g",
      "q", "ngz", "c", "c", "j", "j", "cnz", "j cnz", "njz", "t", "t", "d",
      "d", "n", "ndz", "thz", "thz", "dhz", "dhz", "n", "", "w", "p", "p", "b",
      "b", "m", "mbz", "y", "r", "", "l", "", "", "v", "shz", "shz", "s", "h",
      "l",
      "f", "", "", "", "", "", "", "", "", "aaz", "aez", "awz", "i", "iiz", "u",
      "", "uuz", "", "r u", "e", "eez", "aiz", "o", "ooz", "auz", "l u",
      "r uuz", "l uuz",
      "."};

/////////////////////////////////////////////////////////////////////////////

  private void AcronymsFiller() throws java.io.IOException {
    Acronyms = new ArrayList(1);
    AcronymsExtract = new ArrayList(1);
    UnicodeFileReader ufr = new UnicodeFileReader(AcronymsFilePath);
    UnicodeTokenizer ut;
    while (!ufr.isEnd()) {
      ut = new UnicodeTokenizer(ufr.readLine(), "0009", "uni");
      Acronyms.add(ut.nextToken());
      AcronymsExtract.add(ut.nextToken());
    }
  } //AcronymsFiller()

////////////////////////////////////////////////////////////////////////////

  private void UnitsFiller() throws java.io.IOException {
    Units = new ArrayList(1);
    UnitsDef = new ArrayList(1);
    UnicodeFileReader ufr = new UnicodeFileReader(UnitsFilePath);
    UnicodeTokenizer ut;
    while (!ufr.isEnd()) {
      ut = new UnicodeTokenizer(ufr.readLine(), "0009", "uni");
      Units.add(ut.nextToken());
      UnitsDef.add(ut.nextToken());
    }
  } //AcronymsFiller()

////////////////////////////////////////////////////////////////////////////

  public ArrayList prefixFiller(ArrayList word) throws java.io.IOException{
    ArrayList newWord = new ArrayList(1);
    AcronymsFiller();
    if (Acronyms.contains(word))
      newWord = (ArrayList) AcronymsExtract.get(Acronyms.indexOf(word));
    else
      newWord=word;
    return newWord;
  } //prefix filer()

////////////////////////////////////////////////////////////////////////////

  public ArrayList prefixFiller(String prefix) throws java.io.IOException{
    ArrayList word = new ArrayList(1);
    String item = "";
    for(int i=0; i<prefix.length(); i++) {
      item = item.concat(Character.toString(prefix.charAt(i)));
      if(item.length()==4){
        word.add(item);
        item = "";
      }
    }
    AcronymsFiller();
    if (Acronyms.contains(word))
      word = (ArrayList) AcronymsExtract.get(Acronyms.indexOf(word));
      return word;
  } //prefix filler()

////////////////////////////////////////////////////////////////////////////

  public ArrayList unitsFiller(String sufix) throws java.io.IOException{

    ArrayList word = new ArrayList(1);
    String item = "";
    foundAUnit = false;

    if(sufix.endsWith("002c"))
      sufix = sufix.substring(0, sufix.length() - 4);

    for(int i=0; i<sufix.length(); i++) {
      item = item.concat(Character.toString(sufix.charAt(i)));
      if(item.length()==4){
        word.add(item);
        item = "";
      }
    }

    UnitsFiller();
    if (Units.contains(word)) {
      word = (ArrayList) UnitsDef.get(Units.indexOf(word));
      foundAUnit = true;
    }
      return word;
  } //Unitsfiller()

////////////////////////////////////////////////////////////////////////////

  public void printArray() throws java.io.IOException {
    java.io.RandomAccessFile afr = new java.io.RandomAccessFile(".\\Text\\Data.txt",
        "rw");
    byte enter[] = {
        13, 10};
    for (int i = 0; i < PhoneSet.length; i++) {
      afr.writeBytes(Integer.toString(i));
      afr.writeBytes("\t-\t");
      afr.writeBytes(PhoneSet[i]);
      afr.writeBytes("\t-\t");
      afr.write(enter);
    }
  } //printArray

///////////////////////////////////////////////////////////////////////////

  private ArrayList Wovels() {
    ArrayList wovels = new ArrayList(1);
    wovels.add("0dca");
    wovels.add("0dcf");
    wovels.add("0dd0");
    wovels.add("0dd1");
    wovels.add("0dd2");
    wovels.add("0dd3");
    wovels.add("0dd4");
    wovels.add("0dd6");
    wovels.add("0dd8");
    wovels.add("0dd9");
    wovels.add("0dda");
    wovels.add("0ddb");
    wovels.add("0ddc");
    wovels.add("0ddd");
    wovels.add("0dde");
    wovels.add("0ddf");
    wovels.add("0df2");
    wovels.add("0df3");
    return wovels;
  } //Wovels()

///////////////////////////////////////////////////////////////////////////

  private ArrayList Punctuations() {
    ArrayList punc = new ArrayList(1);
    punc.add(".");
    punc.add(";");
    punc.add(":");
    punc.add(",");
    punc.add("!");
    punc.add(" ");
    punc.add("'");

    return punc;
  } //Wovels()

////////////////////////////////////////////////////////////////////////////

  private ArrayList ValidList() {
    ArrayList valids = new ArrayList(1);
    valids.add("002f");
    valids.add("005c");
    valids.add("005f");
    //valids.add("002d");
    valids.add("003a");
    valids.add("002e");
    valids.add("2013");
    return valids;
  } //Wovels()

////////////////////////////////////////////////////////////////////////////

  public String diphoneSet(ArrayList word) throws java.io.IOException {

    ArrayList ASCIIWord = new ArrayList(1);
    ArrayList wovels = Wovels();
    int ArrayIndex = 0;
    int index = 0;

    AcronymsFiller();

    if (Acronyms.contains(word))
      word = (ArrayList) AcronymsExtract.get(Acronyms.indexOf(word));

    while (word.size() != index++)
      if (word.contains("200d"))
        word.remove("200d");

    for (int i = 0; i < word.size(); i++) {
      if (word.get(i).toString().startsWith("0d")) {
        ArrayIndex = Integer.valueOf(word.get(i).toString().substring(2, 4),
                                     16).intValue();
        index = (ArrayIndex > 224) ? ArrayIndex - 148 : ArrayIndex - 130;
        if (index == 11 || index == 12 || index == 35 || index == 86 ||
            index == 94 || index == 95) {
          ASCIIWord.add(Character.toString(PhoneSet[index].
                                           charAt(0)));
          ASCIIWord.add(PhoneSet[index].substring(2,
                                                  PhoneSet[index].length()));
        }
        else
        if (index == 13 || index == 14) {
          ASCIIWord.add(Character.toString(PhoneSet[index].
                                           charAt(0)));
          ASCIIWord.add(PhoneSet[index].substring(2, 3));
          ASCIIWord.add(PhoneSet[index].substring(4,
                                                  PhoneSet[index].length()));
        }
        else
        if (index != 72)
          ASCIIWord.add(PhoneSet[index]);
        if (22 < index && index < 69)
          if (i < word.size() - 1 &&
              !wovels.contains(word.get(i + 1).toString()) ||
              i == word.size() - 1)
            ASCIIWord.add(SCHWA_SOUND);
      }
      else {
        index = Integer.valueOf(word.get(i).toString().substring(
            2, 4), 16).intValue();
        ASCIIWord.add(Character.toString( (char) index));
      }

    } //for for word

    // Rule 2 & 5

    if (ASCIIWord.size() > 2)
      normalizer(ASCIIWord);

    String listToWord = "";

    for (int i = 0; i < ASCIIWord.size(); i++)
      listToWord = listToWord.concat(ASCIIWord.get(i).toString());

    return listToWord;
  } //diphoneSet()

////////////////////////////////////////////////////////////////////////

  public Element NumDetecter(ArrayList word) throws java.io.IOException {

    ArrayList toNumaric = new ArrayList(1);
    ArrayList toSymbols = new ArrayList(1);
    ArrayList sufixFill = new ArrayList(1);

    String numBuffer = "";
    String prefix = "";
    String sufix = "";

    int PrefixIndex = 0;
    int sufixIndex = word.size()-1;

    boolean isValid = true;
    boolean precentage = false;
    boolean symbolSwitch = false;

    while(true) {
      if (word.get(PrefixIndex).toString().startsWith("003") &&
          Character.isDigit(word.get(PrefixIndex).toString().charAt(3)))
        break;
      prefix = prefix.concat(word.get(PrefixIndex++).toString());
    }// prefix Detecter

    while (true) {
      if (word.get(sufixIndex).toString().startsWith("003") &&
          Character.isDigit(word.get(sufixIndex).toString().charAt(3)))
        break;
      sufix = word.get(sufixIndex--).toString().concat(sufix);
    } // sufix Detecter

    if(sufix.equals("002f003d"))
       sufix = "";

    if(sufix.length()!=0 && sufix.charAt(1)!='d') {
      sufixFill = unitsFiller(sufix);
      if (foundAUnit) {
        sufix = "";
        prefix = "";
        for (int i = sufixFill.size() - 1; i >= 0; i--)
          prefix = sufixFill.get(i).toString().concat(prefix);
      }
    }

   for (int i = PrefixIndex; i <= sufixIndex; i++) {

      if (word.get(i).toString().equals("002c"))
         continue;

      if (word.get(i).toString().startsWith("003") &&
          Character.isDigit(word.get(i).toString().charAt(3))) {
        numBuffer = numBuffer.concat( (String.valueOf(word.get(i).toString().
            charAt(3))));
        symbolSwitch = true;
      }
      else {
        toNumaric.add(numBuffer);
        numBuffer = "";
        if (symbolSwitch) {
          toSymbols.add(word.get(i).toString());
          symbolSwitch = false;
        }
        else {
          isValid = false;
          break;
        }
      }
    } //for

    toNumaric.add(numBuffer);

    if(toNumaric.size() > 3) isValid = false;

    if (sufix.startsWith("0025")) {
      if(toSymbols.size()>1) isValid = false;
        else   precentage = true;
      if(sufix.length()<5)
      sufix = "";
      else
        sufix = sufix.substring(4);
    }

    for(int i=0; i<toSymbols.size(); i++)
      if(!ValidList().contains(toSymbols.get(i))) isValid = false;

    return new Element(prefix, toNumaric, toSymbols, sufix, precentage, isValid);
  } //NumDetecter

/////////////////////////////////////////////////////////////////////////

  public ArrayList sufixFixer(ArrayList word, String sufix, int category) {

    switch (category) {
      case 1:
        sufixAdder(word, sufix);
        break;
      case 2:
        if (word.get(word.size() - 1).equals("0db1") ||
            word.get(word.size() - 1).equals("0dad"))
          word.add("0dca");
        else if (word.get(word.size() - 1).equals("0dc4")) {
          if (word.get(word.size() - 2).equals("0dc5"))
            word.add(word.size() - 1, "0ddc");
          word.remove(word.size() - 1);
          word.add("0dc3");
          word.add("0dca");
        }
        else if (word.get(word.size() - 2).equals("0dd3"))
          word.set(word.size() - 2, "0dd2");
        else if (word.get(word.size() - 1).equals("0dc0")) {
          word.remove(word.size() - 1);
          if (word.get(word.size() - 1).equals("0ddc")) {
            word.add("0dc3");
            word.add("0dca");
          }
        }
        else if (word.get(word.size() - 1).equals("0dba") && word.size() > 2 &&
                 !word.get(word.size() - 3).equals("0dcf") &&
                 !word.get(word.size() - 3).equals("0020"))
          word.remove(word.size() - 1);
        else if (word.get(word.size() - 1).equals("0d9a"))
          if (word.get(word.size() - 2).equals("0d91"))
            word.add("0dca");
          else
            word.remove(word.size() - 1);
        else if (word.get(word.size() - 1).equals("0dc3")) {
          if (word.get(word.size() - 2).equals("0dca")) {
            word.remove(word.size() - 1);
            word.remove(word.size() - 1);
            word.add("0dd2");
          }
          else
            word.add("0dca");
        }
        sufixAdder(word, sufix);
        break;
      case 3:
        word.add(word.size(), "0dd9");
        sufixAdder(word, sufix);
        break;
      case 4:
        word.add(word.size(), "0dda");
        sufixAdder(word, sufix);
        break;
      default:
        word.add("0df4");
        break;

    } //switch
    return word;
  } //sufixFixer()

////////////////////////////////////////////////////////////////////////////

  public int sufixDetecter(String sufix) {
    if (sufix.charAt(sufix.length() - 1) == '3')
      return 4;
    else if (sufix.charAt(3) == '1' || sufix.charAt(3) == '4')
      return 3;
    else if (sufix.charAt(2) == 'c' || (sufix.charAt(3) == 'f' && sufix.length()>4))
      return 2;
    else
      return 1;
  } //sufixDetecter()

///////////////////////////////////////////////////////////////////////////

  public boolean isWithNum(ArrayList word) {
    boolean found = false;
    int index = 0;
    while (word.size() > index) {
      if (word.get(index).toString().charAt(2) == '3' && Character.isDigit(word.get(index).toString().charAt(3))) {
        found = true;
        break;
      }
      else index++;
    }
    return found;
  } //isWithNum()

///////////////////////////////////////////////////////////////////////////

  public ArrayList StringtoAList(String s) {
    ArrayList word = new ArrayList(1);
    for (int i = 0; i < s.length(); i++)
      word.add(Character.toString(s.charAt(i)));
    return word;
  }//StringtoAList()

///////////////////////////////////////////////////////////////////////////

private void sufixAdder(ArrayList word, String sufix) {
    String element = "";
    for(int i=0; i<sufix.length(); i++) {
      element = element.concat(Character.toString(sufix.charAt(i)));
      if(element.length()==4) {
        word.add(element);
        element = "";
      }
    }
  }//sufixAdder

//////////////////////////////////////////////////////////////////////////

  private boolean isVowel(String sufix) {
      ArrayList vowelList = new ArrayList(1);
      vowelList.add("axz");
      for(int i=3; i<21; i++)
        vowelList.add(PhoneSet[i]);
        if(vowelList.contains(sufix))
          return true;
        else return false;
    }//sufixAdder

//////////////////////////////////////////////////////////////////////////

    private void normalizer(ArrayList list) {

      boolean checker = false;
      int i=list.size()-1;
      int counter =0;

      for(; i>0; i--) {
        if(Punctuations().contains(list.get(i).toString()))
          continue;
        else
          counter++;
        if(counter>2) {
          checker = true;
          break;
        }
      }

      if (checker) {
        i = 0;
        for (; i < list.size() - 2; i++) {
          if (list.get(i).toString().equals("axz") &&
              !isVowel(list.get(i + 1).toString()) &&
              !Punctuations().contains(list.get(i + 1).toString()) &&
              !isVowel(list.get(i + 2).toString()))
            list.set(i, "a");
        }

      i = 0;
      while (!isVowel(list.get(i).toString())) {
        i++;
        if (i == list.size() - 1)
          break;
      } //while

      if (i != 0 && list.get(i).toString().equals("axz")) {
        if (! ((i == 1 && list.get(0).toString().equals("k") &&
               list.get(2).toString().equals("r")) ||
               (i == 2 && !isVowel(list.get(0).toString()) &&
                !isVowel(list.get(1).toString())))) {
          list.set(i, "a");
        }
      }
      }
    } //normalizer

//////////////////////////////////////////////////////////////////////////

 class Element {

    private String prefix = "";
    private ArrayList numaric;
    private ArrayList symbols;
    private String sufix = "";
    private boolean precentage;
    private boolean isValid;


    public Element(String pf, ArrayList num, ArrayList sym, String sf, boolean precen, boolean isvalid) {
      prefix = pf;
      numaric = num;
      symbols = sym;
      sufix = sf;
      precentage = precen;
      isValid = isvalid;
    } //Constuctor

    public String getPrefix() {
      return prefix;
    }

    public ArrayList getNumaric() {
      return numaric;
    }

    public ArrayList getSymbols() {
      return symbols;
    }

    public String getSufix() {
      return sufix;
    }

    public boolean isPrecentage() {
      return precentage;
    }

    public boolean isValid() {
      return isValid;
    }

    public String toString() {
      return "\nprefix\t=\t" + prefix + "\n" + "numaric\t=\t" + numaric + "\n" +
          "symbols\t=\t" + symbols + "\n" + "sufix\t=\t" + sufix +
          "\n" + "isPrece\t=\t" + precentage + "\nisValid\t=\t" + isValid;
    }
  } //Element

} //class